
package rankingimagenes;

import com.drew.imaging.ImageMetadataReader;
import com.drew.imaging.ImageProcessingException;
import com.drew.imaging.bmp.BmpMetadataReader;
import com.drew.imaging.gif.GifMetadataReader;
import com.drew.imaging.jpeg.JpegMetadataReader;
import com.drew.imaging.png.PngMetadataReader;
import com.drew.metadata.Directory;
import com.drew.metadata.Metadata;
import com.drew.metadata.Tag;
import java.awt.Color;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

/**
 *
 * @author Diego Vázquez Blanco
 */
public class Utils {
    
    /**
     * Obtiene la extension de un archivo
     * @param f Archivo del cual se va a obtener su extension
     * @return Devuelve la extension del archivo
     */
    public static String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');

        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        }
        return ext;
    }
    
    /**
     * Obtiene los metadatos de la imagen especificada como parametro y los 
     * devuelve como tipo String con etiquetas HTML para usar en un ToolTip.
     * Habría que usarlo del siguiente modo:
     *      JLabel.setToolTip("<html>" + Utils.getMetadata(file) + "</html>");
     * 
     * @param file Imagen del que se van a recoger los metadatos (JPEG, PNG, GIF, BMP)
     * @return Devuelve una cadena tipo String con los metadatos y etiquetas de formato HTML
     */
    public static String getMetadata(File file){
        String cadena = "";
        String extension = Utils.getExtension(file);
        Metadata metadata;
        
        try {
            switch (extension) {
                case "jpeg":
                    metadata = JpegMetadataReader.readMetadata(file);
                    break;
                case "png":
                    metadata = PngMetadataReader.readMetadata(file);
                    break;
                case "gif":
                    metadata = GifMetadataReader.readMetadata(file);
                    break;
                case "bmp":
                    metadata = BmpMetadataReader.readMetadata(file);
                    break;
                default:
                    metadata = ImageMetadataReader.readMetadata(file);
                    break;
            }
            
            // Itera sobre los datos
            // Un objeto Metadata contiene multiples objetos Directory
            for (Directory directory : metadata.getDirectories()) {
                   
                cadena = "<p>"; // Encerramos entre <p> y </p> para que 
                                // solo salga Nombre, Tamano, FechMod
                // Cada Directory guarda valores en los objetos Tag
                for (Tag tag : directory.getTags()) {
                    
                    if (!extension.equals("bmp")){
                        try{cadena += tag.getTagName().substring(5) + ": ";
                        }catch(IndexOutOfBoundsException e){}
                    } 
                    else{
                        cadena += tag.getTagName() + ": ";
                    }
                    cadena += tag.getDescription() + "<br>";
                }
                
                cadena += "</p>";

                // Un Directory también puede contener mensajes de error
                if (directory.hasErrors()) {
                    for (String error : directory.getErrors()) {
                        System.err.println("ERROR en getMetadata("+ file +"): " + error);
                    }
                }
            }
        } catch (ImageProcessingException | IOException e) {}
        return cadena;
    }
    
    /**
     * Calculamos la media de una variable Color
     * @param color Color del cual se quiere obtener la media
     * @return entero con el valor de la media
     */
    private static int calcularMedia(Color color){
        int mediaColor;
        mediaColor=(int)((color.getRed()+color.getGreen()+color.getBlue())/3);
        return mediaColor;
    }
    
    /**
     * Devuelve el histograma de la imagen.
     * @param imagen BufferedImagen de la cual se quiere obtener el histograma
     * @return Devuelve una variable int[5][256], donde el primer campo[0] corresponde
     * al canal Rojo, [1]=verde [2]=azul [3]=alfa [4]=escala grises
     */
    public static int[][] getHistograma(BufferedImage imagen){
        Color colorAuxiliar;
        /* Creamos la variable que contendra el histograma
        El primer campo [0], almacenara el histograma Rojo
        [1]=verde [2]=azul [3]=alfa [4]=escala grises */
        int histogramaReturn[][]=new int[5][256];
        // Recorremos la imagen 
        for( int i = 0; i < imagen.getWidth(); i++ ){
            for( int j = 0; j < imagen.getHeight(); j++ ){
                // Obtenemos color del píxel actual
                colorAuxiliar=new Color(imagen.getRGB(i, j));
                // Sumamos una unidad en la fila roja [0], 
                // en la columna del color rojo obtenido
                histogramaReturn[0][colorAuxiliar.getRed()]+=1;
                histogramaReturn[1][colorAuxiliar.getGreen()]+=1;
                histogramaReturn[2][colorAuxiliar.getBlue()]+=1;
                histogramaReturn[3][colorAuxiliar.getAlpha()]+=1;
                histogramaReturn[4][calcularMedia(colorAuxiliar)]+=1;
            }
        }
        return histogramaReturn;
    }
}
